@@ -1,5 +1,6 @@
 import inspect
 from functools import lru_cache
+from io import TextIOWrapper
 from typing import Sequence, Dict, Optional
 
 from black import Mode, format_str
@@ -10,6 +11,14 @@ from expr_codegen.expr import get_current_by_prefix, get_children, replace_exprs
 from expr_codegen.model import dag_start, dag_end, dag_middle
 
 
+def simplify2(expr):
+    try:
+        expr = simplify(expr)
+    except AttributeError as e:
+        print(f'{expr} ,表达式无法简化, {e}')
+    return expr
+
+
 class ExprTool:
 
     def __init__(self):
@@ -37,7 +46,7 @@ class ExprTool:
 
         """
         # 抽取前先化简
-        expr = simplify(expr)
+        expr = simplify2(expr)
 
         exprs = []
         syms = []
@@ -86,9 +95,9 @@ class ExprTool:
 
         # 不做改动，直接生成
         for variable, expr in repl:
-            exprs_dict[variable] = simplify(expr)
+            exprs_dict[variable] = simplify2(expr)
         for variable, expr in redu:
-            exprs_dict[variable] = simplify(expr)
+            exprs_dict[variable] = simplify2(expr)
 
         return exprs_dict
 
@@ -134,7 +143,7 @@ class ExprTool:
 
         return self.exprs_dict
 
-    def dag(self, merge):
+    def dag(self, merge: bool):
         """生成DAG"""
         G = dag_start(self.exprs_dict, self.get_current_func, self.get_current_func_kwargs)
         if merge:
@@ -223,47 +232,83 @@ class ExprTool:
         return globals_['df_output']
 
     @lru_cache(maxsize=64)
-    def _get_codes(self, source: str, extra_codes: str, output_file: str) -> str:
+    def _get_code(self,
+                  source: str, *more_sources: str,
+                  extra_codes: str, output_file: str,
+                  style='polars', template_file='template.py.j2',
+                  date='date', asset='asset') -> str:
         """通过字符串生成代码， 加了缓存，多次调用不重复生成"""
-        raw, exprs_dict = sources_to_exprs(self.globals_, source, safe=False)
+        raw, exprs_dict = sources_to_exprs(self.globals_, source, *more_sources)
 
         # 生成代码
-        codes, G = _TOOL_.all(exprs_dict, style='polars', template_file='template.py.j2',
-                              replace=True, regroup=True, format=True,
-                              date='date', asset='asset',
-                              # 复制了需要使用的函数，还复制了最原始的表达式
-                              extra_codes=(raw,
-                                           # 传入多个列的方法
-                                           extra_codes,
-                                           ))
-
-        if output_file is not None:
+        code, G = _TOOL_.all(exprs_dict, style=style, template_file=template_file,
+                             replace=True, regroup=True, format=True,
+                             date=date, asset=asset,
+                             # 复制了需要使用的函数，还复制了最原始的表达式
+                             extra_codes=(raw,
+                                          # 传入多个列的方法
+                                          extra_codes,
+                                          ))
+        if isinstance(output_file, TextIOWrapper):
+            output_file.write(code)
+        elif output_file is not None:
             with open(output_file, 'w', encoding='utf-8') as f:
-                f.write(codes)
+                f.write(code)
 
-        return codes
+        return code
 
 
 _TOOL_ = ExprTool()
 
 
-def codegen_exec(code_block, df_input,
+def codegen_exec(df,
+                 *codes,
                  extra_codes: str = r'CS_SW_L1 = pl.col(r"^sw_l1_\d+$")',
-                 output_file: Optional[str] = None):
-    """快速转换源代码并执行"""
+                 output_file: Optional[str] = None,
+                 style: str = 'polars', template_file: str = 'template.py.j2',
+                 date: str = 'date', asset: str = 'asset'
+                 ):
+    """快速转换源代码并执行
+
+    Parameters
+    ----------
+    df: pl.DataFrame
+        输入DataFrame
+    codes:
+        函数体。此部分中的表达式会被翻译成目标代码
+    extra_codes: str
+        额外代码。不做处理，会被直接复制到目标代码中
+    output_file: str
+        保存生成的目标代码到文件中
+    style: str
+        代码风格。可选值 ('polars', 'pandas')
+    template_file: str
+        代码模板
+    date: str
+        时间字段
+    asset: str
+        资产字段
+
+    Returns
+    -------
+    pl.DataFrame
+
+    """
     # 此代码来自于sympy.var
     frame = inspect.currentframe().f_back
     _TOOL_.globals_ = frame.f_globals.copy()
     del frame
 
-    if isinstance(code_block, str):
-        source = code_block
-    else:
-        source = inspect.getsource(code_block)
+    more_sources = [c if isinstance(c, str) else inspect.getsource(c) for c in codes]
 
-    codes = _TOOL_._get_codes(source, extra_codes, output_file)
+    code = _TOOL_._get_code(
+        *more_sources, extra_codes=extra_codes,
+        output_file=output_file,
+        style=style, template_file=template_file,
+        date=date, asset=asset,
+    )
 
-    if df_input is None:
-        return df_input
+    if df is None:
+        return df
     else:
-        return _TOOL_.exec(codes, df_input)
+        return _TOOL_.exec(code, df)
